﻿using SharpDX;

namespace Droggelbecher
{
    class Helpers
    {
        /// <summary>
        /// Tests all lines of the triangle against collision with the given circle
        /// </summary>
        /// <param name="center">center of the circle</param>
        /// <param name="radius"></param>
        /// <param name="v1"></param>
        /// <param name="v2"></param>
        /// <param name="v3"></param>
        /// <returns></returns>
        public static bool CircleIntersectsTriangle(Vector2 center, float radius, Vector2 v1, Vector2 v2, Vector2 v3)
        {
            return (CircleIntersectsLine(center, radius, v1, v2)
                    && CircleIntersectsLine(center, radius, v2, v3)
                    && CircleIntersectsLine(center, radius, v3, v1));
        }

        public static bool CircleIntersectsLine(Vector2 center, float radius, Vector2 v1, Vector2 v2)
        {
            return (DistanceBetweenCircleLine(center, radius, v1, v2) <= 0);
        }

        public static float DistanceBetweenCircleLine(Vector2 center, float radius, Vector2 v1, Vector2 v2)
        {
            Vector2 a = center - v1;
            Vector2 b = v2 - v1;

            // rotate b by 90° clockwise to get normal
            Vector2 normal = new Vector2(b.Y, -b.X);
            normal = normal / normal.Length();

            //calculate smallest distance between the circle and the straight line determined by the edge vector
            float distance = Vector2.Dot(a, normal) - radius;

            // projection vector
            Vector2 projection = normal * distance;

            // biggest accepted vector between circle and v1;
            Vector2 biggest = b + projection;

            // test if circle is in edge range
            bool canIntersect = true;
            if (Vector2.Dot(a, b) < 0) canIntersect = false; // out of range on side of v1
            if (Vector2.Dot(-b, center - v2) < 0) canIntersect = false; // out of range on side of v2
            // if (Vector2.Dot(biggest, biggest) > Vector2.Dot(b, b)) canIntersect = false; // out of range on side of v2

            if (canIntersect)
                return distance;
            else return 100;
        }

        /// <summary>
        /// Transformation of a Vector3 without projection.
        /// </summary>
        /// <param name="_v">The vector to transform</param>
        /// <param name="_m">The transformation matrix (rotation, scalation, translation)</param>
        /// <returns>The first 3 components of (_v, 1) * _m</returns>
        public static Vector3 Transform(Vector3 _v, ref Matrix _m)
        {
            Vector4 r;
            Vector3.Transform(ref _v, ref _m, out r);
            return new Vector3(r.X, r.Y, r.Z);
        }


        /// <summary>
        /// Computes the transposed inverse matrix without translation part.
        /// This matrix is usefull for normal-space transformations
        /// </summary>
        /// <param name="_m"></param>
        /// <returns></returns>
        public static Matrix CreateInverseTranspose(ref Matrix _m)
        {
            Matrix transformInvTr = _m;
            transformInvTr.TranslationVector = new Vector3(0.0f, 0.0f, 0.0f);
            transformInvTr.Invert();
            transformInvTr.Transpose();
            return transformInvTr;
        }

        public static Vector3 jumpertoview(JumpNRun jmpnrun)
        {
            switch (jmpnrun.jumper.wall.side)
            {
                case Wall.Side.FRONT:
                    return new Vector3(-jmpnrun.jumper.position.X,jmpnrun.jumper.position.Y,-1);
                    break;
                case Wall.Side.RIGHT:
                    return new Vector3(1,jmpnrun.jumper.position.Y,-jmpnrun.jumper.position.X);
                    break;
                case Wall.Side.BACK:
                    return new Vector3(jmpnrun.jumper.position.X, jmpnrun.jumper.position.Y,1);
                    break;
                case Wall.Side.LEFT:
                    return new Vector3(-1,jmpnrun.jumper.position.Y,jmpnrun.jumper.position.X);
                    break;
                case Wall.Side.UP:
                    return new Vector3(jmpnrun.jumper.position.X,1,jmpnrun.jumper.position.Y);
                    break;
                case Wall.Side.DOWN:
                    return new Vector3(jmpnrun.jumper.position.Y,-1,jmpnrun.jumper.position.X);
                    break;

            }
            return new Vector3();
        }
    }
}
