﻿/**
 * Here you are:
 * True Game-Jam Code!
 * 
 * */


using SharpDX;
using SharpDX.Toolkit;
using SharpDX.Toolkit.Graphics;
using Assimp;
using System;
using SharpDX.Toolkit.Input;
using System.Collections.Generic;

namespace Droggelbecher
{
    class World : Game
    {
        DataStream vertexStream;
        VertexInputLayout InputLayout;
        int vertexSize;
        private GraphicsDeviceManager m_graphicsDeviceManager;
        private Model model;
        private static Effect m_simpleEffect;
        //private Effect m_shadowEffect;
        private BasicEffect shadowEffect;

        public static Vector3 cameraPositionGetter
        {
            get { return _cameraPosition; }
        }
        public Vector3 cameraPosition
        {
            get { return _cameraPosition; }
            set
            {
                _cameraPosition = value;
                _view = Matrix.LookAtRH(
                    _cameraPosition,            // Position
                    Vector3.Zero,               // At (point which is centered in the middle of the screen).
                    new Vector3(0.0f, 1.0f, 0.0f)); // Up
                _view.Row1 = -_view.Row1;
                _viewProjection = _view * _projection;
            }
        }
        private static Vector3 _cameraPosition;
        private Matrix _view;
        private Matrix _projection;
        private Matrix _viewProjection;
        private Matrix projection
        {
            get { return _projection; }
            set
            {
                _projection = value;
                _viewProjection = _view * _projection;
            }
        }

        //private SpriteBatch spriteBatch;

        KeyboardManager keyboardManager;
        KeyboardState keyboardState;
        KeyboardState previousKeyboardState;
        MouseManager mouseManager;
        MouseState mouseState;
        MouseState previousMouseState;

        Menu menu;
        OuterCube outerCube;
        OuterCube[] levels;
        int level = 0;
        JumpNRun jumpNRun;

        RasterizerState backfaceCullingState;
        RasterizerState frontfaceCullingState;
        RasterizerState noneCullingState; // for testing
        DepthStencilState depthStencilStateState;
        SamplerState linearSamplerState;
        BlendState blendStateOpaque;
        private VertexInputLayout shadowInputLayout;

        /// <summary>
        /// Entry point create and starts the game
        /// </summary>
        static void Main()
        {
            var program = new World();
            program.Run();
        }

        World()
        {
            // Creates a graphics manager. This is mandatory.
            m_graphicsDeviceManager = new GraphicsDeviceManager(this);
            m_graphicsDeviceManager.PreferredDepthStencilFormat = DepthFormat.Depth32;
            m_graphicsDeviceManager.SetPreferredGraphicsProfile(SharpDX.Direct3D.FeatureLevel.Level_10_1);

            // Setup the relative directory to the executable directory
            // for loading contents with the ContentManager
            Content.RootDirectory = "content";
        }

        

        protected override void Initialize()
        {
            base.Initialize();

            Window.Title = "Lightception";
            menu = new Menu();
            keyboardManager = new KeyboardManager(this);
            mouseManager = new MouseManager(this);
            LoadLevel();

            

            jumpNRun = new JumpNRun(outerCube.sideWalls);

            base.Window.AllowUserResizing = true;

            // Setup Camera
            cameraPosition = new Vector3(0.0f, 0.5f, -2.0f); // updates view-Matrix

            projection = Matrix.PerspectiveFovRH(
                0.7f,                               // Field of view
                (float)GraphicsDevice.BackBuffer.Width / GraphicsDevice.BackBuffer.Height,  // Aspect ratio
                0.5f,                               // Near clipping plane
                200.0f);                            // Far clipping plane
            // viewProjection = view * projection; no more needed, because Properties

            // Setup states
            var rasterizerStateDesc = SharpDX.Direct3D11.RasterizerStateDescription.Default();
            backfaceCullingState = RasterizerState.New(GraphicsDevice, "CullModeBack", rasterizerStateDesc);

            rasterizerStateDesc.CullMode = SharpDX.Direct3D11.CullMode.Front;
            frontfaceCullingState = RasterizerState.New(GraphicsDevice, "CullModeFront", rasterizerStateDesc);

            // for testing
            rasterizerStateDesc.CullMode = SharpDX.Direct3D11.CullMode.None;
            noneCullingState = RasterizerState.New(GraphicsDevice, "CullModeNone", rasterizerStateDesc);

            var depthStencilStateDesc = SharpDX.Direct3D11.DepthStencilStateDescription.Default();
            depthStencilStateState = DepthStencilState.New(GraphicsDevice, "NormalZBufferUse", depthStencilStateDesc);

            var samplerStateDesc = SharpDX.Direct3D11.SamplerStateDescription.Default();
            samplerStateDesc.AddressV = SharpDX.Direct3D11.TextureAddressMode.Wrap;
            samplerStateDesc.AddressU = SharpDX.Direct3D11.TextureAddressMode.Wrap;
            linearSamplerState = SamplerState.New(GraphicsDevice, "LinearSampler", samplerStateDesc);

            var blendStateDesc = SharpDX.Direct3D11.BlendStateDescription.Default();
            blendStateOpaque = BlendState.New(GraphicsDevice, "Opaque", blendStateDesc);
        }

        protected override void LoadContent()
        {
            // Importer for many models
            var importer = new AssimpImporter();

            // Load a specific model
            string fileName = System.IO.Path.GetFullPath(Content.RootDirectory + "/cube.3ds");
            Scene scene = importer.ImportFile(fileName, PostProcessSteps.MakeLeftHanded);
            model = new Model(scene, GraphicsDevice, Content);

            // Load shader
            EffectCompilerFlags compilerFlags = EffectCompilerFlags.None;
            EffectCompiler compiler = new EffectCompiler();
#if DEBUG
            compilerFlags |= EffectCompilerFlags.Debug;
#endif

            var simpleShaderCompileResult = compiler.CompileFromFile(Content.RootDirectory + "/simple.fx", compilerFlags);
            if (simpleShaderCompileResult.HasErrors)
            {
                System.Console.WriteLine(simpleShaderCompileResult.Logger.Messages);
                System.Diagnostics.Debugger.Break();
            }
            m_simpleEffect = new SharpDX.Toolkit.Graphics.Effect(GraphicsDevice, simpleShaderCompileResult.EffectData);
            m_simpleEffect.Parameters["diffuseSampler"].SetResource(linearSamplerState);

            // shadow shader
            /*
            var shadowShaderCompileResult = compiler.CompileFromFile(Content.RootDirectory + "/shadow.fx", compilerFlags);
            if (shadowShaderCompileResult.HasErrors)
            {
                System.Console.WriteLine(shadowShaderCompileResult.Logger.Messages);
                System.Diagnostics.Debugger.Break();
            }

            m_shadowEffect = new SharpDX.Toolkit.Graphics.Effect(GraphicsDevice, shadowShaderCompileResult.EffectData);
            //m_simpleEffect.Parameters["diffuseSampler"].SetResource(m_linearSamplerState);
            */

            // Creates a basic effect
            shadowEffect = ToDisposeContent(new BasicEffect(GraphicsDevice)
            {
                VertexColorEnabled = true,
                View = Matrix.Identity,
                Projection = Matrix.Identity,
                World = Matrix.Identity
                //Alpha = 0.4f
            });
            // m_graphicsDeviceManager.GraphicsDevice.SetBlendState(GraphicsDevice.BlendStates.AlphaBlend);
            //SmallCube.LoadContent(GraphicsDevice, Content);

        }

        protected override void Update(GameTime gameTime)
        {
            // end game
            if (keyboardState.IsKeyDown(Keys.Escape))
            {
                this.Exit();
            }

            base.Update(gameTime);
            menu.update(keyboardState, gameTime);
            keyboardState = keyboardManager.GetState();
            mouseState = mouseManager.GetState();
            float rotationAmount = 0.2f;
            float upDownTranslationAmount = 0.1f;
            Vector2 mouseChange = new Vector2(mouseState.X - 0.5f, mouseState.Y - 0.5f);    // same random value != (0,0)

            if (!keyboardState.IsKeyDown(Keys.I))
            {
                cameraPosition = Vector3.Add(new Vector3(0, 0, 0), -Helpers.jumpertoview(jumpNRun)) * new Vector3(-1,1,1);
                /*
                cameraPosition = (Vector3)Vector3.Transform(cameraPosition, Matrix.RotationY(-mouseChange.X * rotationAmount));
                cameraPosition = (Vector3)Vector3.Transform(cameraPosition, Matrix.Translation(Vector3.Up * mouseChange.Y * upDownTranslationAmount));
                int mouseWheelChange = mouseState.WheelDelta - previousMouseState.WheelDelta;
                cameraPosition *= (1 - mouseWheelChange * 0.001f);
                //cameraPosition = (Vector3)Vector3.Transform(cameraPosition, Matrix.RotationZ(mouseState.WheelDelta));
                mouseManager.SetPosition(Vector2.One * 0.5f); // kind of random value != (0,0)
                 */
            }

            outerCube.update(keyboardState, gameTime, jumpNRun.jumper);

            jumpNRun.update(keyboardState);

            previousMouseState = mouseState;
            previousKeyboardState = keyboardState;

            if (outerCube.isWon)
            {
                if (level >= levels.Length - 1)
                {
                    Console.WriteLine("YOU WON THE GAME!\nCongratulations!");
                    throw new Exception("YOU WON THE GAME!\nCongratulations!");
                }
                outerCube = levels[++level];
                AssembleCubes(outerCube.cubes);
                jumpNRun = new JumpNRun(outerCube.sideWalls);
                // Setup Camera
                cameraPosition = new Vector3(0.0f, 0.5f, -2.0f); // updates view-Matrix
            }

        }

        protected void DrawBox(GameTime gameTime)
        {
            //spriteBatch.Begin(SpriteSortMode.Deferred, m_simpleEffect);
            //m_simpleEffect.Parameters["diffuseSampler"].SetResource(linearSamplerState);

            GraphicsDevice.SetDepthStencilState(depthStencilStateState);
            GraphicsDevice.SetBlendState(blendStateOpaque);

            // Slow rotating light
            //double angle = -gameTime.TotalGameTime.TotalMilliseconds / 3000.0;
            m_simpleEffect.ConstantBuffers["Transforms"].Parameters["lightPos"].SetValue(outerCube.getLightPostion());


            // Defines the transformation for the next model to be drawn
            // Matrix transformation = Matrix.RotationZ((float)gameTime.TotalGameTime.TotalMilliseconds / 1000.0f);
            // Draws the model
            //drawModel(model, GraphicsDevice, simpleEffect, transformation);
            drawModel(model, GraphicsDevice, Matrix.Identity, _viewProjection);

            //model.Draw(m_graphicsDeviceManager.GraphicsDevice);

            base.Draw(gameTime);

            // spriteBatch.End();
        }

        protected override void Draw(GameTime gameTime)
        {

            // Clears the screen
            GraphicsDevice.Clear(new Color(0,0,0));

            GraphicsDevice.SetRasterizerState(backfaceCullingState);
            m_simpleEffect.CurrentTechnique.Passes[0].Apply();
            if (menu.menu == false)
            {
                DrawBox(gameTime);

                shadowEffect.CurrentTechnique.Passes[0].Apply();
                GraphicsDevice.SetRasterizerState(noneCullingState);


                List<Shadow> shadows = outerCube.getAllShadows();

                ShadowHexagon[] pointerHex = (ShadowHexagon[])jumpNRun.jumper;
                ShadowHexagon[] shadowHex = new ShadowHexagon[shadows.Count + pointerHex.Length];
                for (int j = 0; j < pointerHex.Length; ++j)
                    shadowHex[j] = pointerHex[j];
                int i = pointerHex.Length;

                foreach (Shadow shadow in shadows)
                {
                    // I LOVE this cast :3
                    shadowHex[i++] = (ShadowHexagon)shadow;
                }
                //this should be backface culling... well, shit on it!
                //ShadowHexagon[] shadowHex = new ShadowHexagon[1];
                DrawShadowAndLight(shadowHex, outerCube.getGoalHex(), outerCube.light, gameTime);


                base.Draw(gameTime);
                GraphicsDevice.SetRasterizerState(frontfaceCullingState);
                m_simpleEffect.CurrentTechnique.Passes[0].Apply();
                List<SmallCube> cubes = new List<SmallCube>(outerCube.cubes.Count);
                foreach (SmallCube cube in outerCube.cubes)
                {
                    cubes.Add(cube); //cube.draw(m_graphicsDeviceManager.GraphicsDevice, _viewProjection, m_simpleEffect);
                }
                this.AssembleCubes(cubes);
                this.DrawCubes(cubes.Count);

                base.Draw(gameTime);
            }
        }


        /// <summary>
        /// Helper-Function to draw a Model.
        /// Transformation: first Scaling, then Rotation, then Translation.
        /// </summary>
        /// <param name="model">model to be drawn</param>
        /// <param name="graphicsDevice">GraphicsDevice</param>
        /// <param name="effect">Effect</param>
        /// <param name="scale">Scale of the model</param>
        /// <param name="rotation">Rotation of the model</param>
        /// <param name="translation">Translation of the model</param>
        public static void drawModel(Model model, GraphicsDevice graphicsDevice, Matrix scale, Matrix rotation, Matrix translation, Matrix viewProjection)
        {
            drawModel(model, graphicsDevice, scale * rotation * translation, viewProjection);
        }

        /// <summary>
        /// Helper-Function to draw a Model.
        /// </summary>
        /// <param name="model">model to be drawn</param>
        /// <param name="graphicsDevice">GraphicsDevice</param>
        /// <param name="effect">Effect</param>
        /// <param name="transformation">Transformation of the model</param>
        public static void drawModel(Model model, GraphicsDevice graphicsDevice, Matrix transformation, Matrix viewProjection)
        {
            // Fill the one constant buffer. Hint: it is not necessary to set
            // things which did not change each frame. But in our case everything
            // is changing

            var transformCB = m_simpleEffect.ConstantBuffers["Transforms"];
            transformCB.Parameters["worldViewProj"].SetValue(transformation * viewProjection);
            transformCB.Parameters["world"].SetValue(transformation);
            Matrix worldInvTr = Helpers.CreateInverseTranspose(ref transformation);
            transformCB.Parameters["worldInvTranspose"].SetValue(worldInvTr);

            // Draw model
            model.Draw(graphicsDevice, m_simpleEffect);
        }



        public class ShadowHexagon
        {
            public Vector2[] points;
            public Droggelbecher.Wall.Side side;
            static float eps = 0.02f;


            public ShadowHexagon(Vector2 pp0, Vector2 pp1, Vector2 pp2, Vector2 pp3, Vector2 pp4, Vector2 pp5, Wall.Side side)
            {
                points = new Vector2[6];
                points[0] = pp0;
                points[1] = pp1;
                points[2] = pp2;
                points[3] = pp3;
                points[4] = pp4;
                points[5] = pp5;
                this.side = side;
            }

            public ShadowHexagon(Vector2[] points, Wall.Side side)
            {
                if (points.Length != 6)
                    throw new Exception("Noooo!");
                this.points = points;
                this.side = side;
            }

            public Vector3 getVec3(int index)
            {
                Vector3 output;
                Vector2 point = points[index];
                switch (side)
                {
                    case Wall.Side.BACK:
                        output = new Vector3(point.X, point.Y, 1 - eps);
                        break;
                    case Wall.Side.FRONT:
                        output = new Vector3(-point.X, point.Y, -1 + eps);
                        break;

                    case Wall.Side.RIGHT:
                        output = new Vector3(1 - eps, point.Y, -point.X);
                        break;
                    case Wall.Side.LEFT:
                        output = new Vector3(-1 + eps, point.Y, point.X);
                        break;

                    default:
                        output = new Vector3(0);
                        break;

                }
                return output;
            }

        }


        private void DrawShadowAndLight(ShadowHexagon[] shadow, ShadowHexagon goalHex, Light light, GameTime gameTime)
        {
            VertexPositionColor[] data = new VertexPositionColor[4 * 3 * shadow.Length + 6 + 12];

            Color shadowGrey = new Color(0.1f, 0.1f, 0.1f, 0.4f);

            for (int index = 0; index < shadow.Length; ++index)
            {
                int i = index * 12;
                data[i++] = new VertexPositionColor(shadow[index].getVec3(0), shadowGrey);
                data[i++] = new VertexPositionColor(shadow[index].getVec3(1), shadowGrey);
                data[i++] = new VertexPositionColor(shadow[index].getVec3(2), shadowGrey);

                data[i++] = new VertexPositionColor(shadow[index].getVec3(0), shadowGrey);
                data[i++] = new VertexPositionColor(shadow[index].getVec3(2), shadowGrey);
                data[i++] = new VertexPositionColor(shadow[index].getVec3(5), shadowGrey);

                data[i++] = new VertexPositionColor(shadow[index].getVec3(2), shadowGrey);
                data[i++] = new VertexPositionColor(shadow[index].getVec3(3), shadowGrey);
                data[i++] = new VertexPositionColor(shadow[index].getVec3(5), shadowGrey);

                data[i++] = new VertexPositionColor(shadow[index].getVec3(3), shadowGrey);
                data[i++] = new VertexPositionColor(shadow[index].getVec3(4), shadowGrey);
                data[i++] = new VertexPositionColor(shadow[index].getVec3(5), shadowGrey);

            }
            // put in goalHex
            Color shinyGoal = new Color(1, (float)144 / 256, (float)1 / 256, 1.0f);
            int j = 12 * shadow.Length;
            data[j++] = new VertexPositionColor(goalHex.getVec3(0), shinyGoal);
            data[j++] = new VertexPositionColor(goalHex.getVec3(2), shinyGoal);
            data[j++] = new VertexPositionColor(goalHex.getVec3(4), shinyGoal);

            data[j++] = new VertexPositionColor(goalHex.getVec3(1), shinyGoal);
            data[j++] = new VertexPositionColor(goalHex.getVec3(3), shinyGoal);
            data[j++] = new VertexPositionColor(goalHex.getVec3(5), shinyGoal);

            /////////// LIGHT PYRAMID
            Vector3[] pyramid = new Vector3[4];
            float third = (float)1/3;
            float scale = 0.05f;
            pyramid[0] = new Vector3(-0.5f, -third, third) * scale + light.position;
            pyramid[1] = new Vector3(0, -third, -2*third) * scale + light.position;
            pyramid[2] = new Vector3(0.5f, -third, third) * scale + light.position;
            pyramid[3] = new Vector3(0, 2 * third, 0) * scale + light.position;

            data[j++] = new VertexPositionColor(pyramid[0], shinyGoal);
            data[j++] = new VertexPositionColor(pyramid[1], shinyGoal);
            data[j++] = new VertexPositionColor(pyramid[3], shinyGoal);

            data[j++] = new VertexPositionColor(pyramid[0], shinyGoal);
            data[j++] = new VertexPositionColor(pyramid[2], shinyGoal);
            data[j++] = new VertexPositionColor(pyramid[1], shinyGoal);

            data[j++] = new VertexPositionColor(pyramid[0], shinyGoal);
            data[j++] = new VertexPositionColor(pyramid[3], shinyGoal);
            data[j++] = new VertexPositionColor(pyramid[2], shinyGoal);

            data[j++] = new VertexPositionColor(pyramid[1], shinyGoal);
            data[j++] = new VertexPositionColor(pyramid[2], shinyGoal);
            data[j++] = new VertexPositionColor(pyramid[3], shinyGoal);

            Buffer<VertexPositionColor> buffer = ToDisposeContent(SharpDX.Toolkit.Graphics.Buffer.Vertex.New(
                GraphicsDevice,
                data));
            ToDisposeContent(buffer);

            // Create an input layout from the vertices
            shadowInputLayout = VertexInputLayout.FromBuffer(0, buffer);

            // Setup the vertices
            GraphicsDevice.SetVertexBuffer(buffer);
            GraphicsDevice.SetVertexInputLayout(shadowInputLayout);

            shadowEffect.Projection = projection;
            shadowEffect.View = _view;
            shadowEffect.Alpha = 0.5f;

            // Apply the basic effect technique and draw the rotating cube
            GraphicsDevice.Draw(SharpDX.Toolkit.Graphics.PrimitiveType.TriangleList, buffer.ElementCount);

            // Handle base.Draw
            //base.Draw(gameTime);
        }

        public void AssembleCubes(List<SmallCube> cubes)
        {
            int numCubes = cubes.Count;

            //TextureSlot texture = material.GetTexture(TextureType.Diffuse, 0);
            // Create new texture for mesh
            var dxtexture = Content.Load<Texture2D>("DOLPHIN.PNG");
            var dxtextureNorm = Content.Load<Texture2D>("normal_wurschtl.png");

            m_simpleEffect.Parameters["diffuseTexture"].SetResource(dxtexture);
            m_simpleEffect.Parameters["normalTexture"].SetResource(dxtextureNorm);
            //modelMesh.DiffuseTexture = dxtexture;
            //modelMesh.NormalTexture = dxtextureNorm;


            int numElements = 1 + 1 + 0 + 1 + 0 + 0;

            // Create vertex element list: Here starts the section of creating SharpDX stuff
            VertexElement[] vertexElements = new VertexElement[numElements];
            uint elementIndex = 0;
            vertexElements[elementIndex++] = new VertexElement("POSITION", 0, SharpDX.DXGI.Format.R32G32B32_Float, 0);
            int vertexSize = Utilities.SizeOf<Vector3>();

            vertexElements[elementIndex++] = new VertexElement("NORMAL", 0, SharpDX.DXGI.Format.R32G32B32_Float, vertexSize);
            vertexSize += Utilities.SizeOf<Vector3>();


            vertexElements[elementIndex++] = new VertexElement("TEXCOORD", 0, SharpDX.DXGI.Format.R32G32_Float, vertexSize);
            vertexSize += Utilities.SizeOf<Vector2>();


            // Set the vertex elements and size
            InputLayout = VertexInputLayout.New(VertexBufferLayout.New(0, vertexElements));
            this.vertexSize = vertexSize;


            // Create data stream for vertices
            //System.IO.MemoryStream vertexStream = new System.IO.MemoryStream(mesh.VertexCount * vertexSize);
            vertexStream = new DataStream(12 * 3 * numCubes * vertexSize, true, true); //change 6 to 12 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


            foreach (SmallCube cube in cubes)
            {
                Vector3[] corners = cube.getCorners();

                /// FRONT ///
                // pos
                vertexStream.Write<Vector3>(corners[0]);
                // normal
                vertexStream.Write<Vector3>(new Vector3(0, 0, -1));
                // uv
                vertexStream.Write<Vector2>(new Vector2(0, 0));

                // pos
                vertexStream.Write<Vector3>(corners[1]);
                // normal
                vertexStream.Write<Vector3>(new Vector3(0, 0, -1));
                // uv
                vertexStream.Write<Vector2>(new Vector2(1, 0));

                // pos
                vertexStream.Write<Vector3>(corners[2]);
                // normal
                vertexStream.Write<Vector3>(new Vector3(0, 0, -1));
                // uv
                vertexStream.Write<Vector2>(new Vector2(1, 1));

                //2 nd

                // pos
                vertexStream.Write<Vector3>(corners[0]);
                // normal
                vertexStream.Write<Vector3>(new Vector3(0, 0, -1));
                // uv
                vertexStream.Write<Vector2>(new Vector2(0, 0));

                // pos
                vertexStream.Write<Vector3>(corners[2]);
                // normal
                vertexStream.Write<Vector3>(new Vector3(0, 0, -1));
                // uv
                vertexStream.Write<Vector2>(new Vector2(1, 1));

                // pos
                vertexStream.Write<Vector3>(corners[3]);
                // normal
                vertexStream.Write<Vector3>(new Vector3(0, 0, -1));
                // uv
                vertexStream.Write<Vector2>(new Vector2(0, 1));

                /// LEFT ///
                // pos
                vertexStream.Write<Vector3>(corners[0]);
                // normal
                vertexStream.Write<Vector3>(new Vector3(-1, 0, 0));
                // uv
                vertexStream.Write<Vector2>(new Vector2(1, 0));

                // pos
                vertexStream.Write<Vector3>(corners[3]);
                // normal
                vertexStream.Write<Vector3>(new Vector3(-1, 0, 0));
                // uv
                vertexStream.Write<Vector2>(new Vector2(1, 1));

                // pos
                vertexStream.Write<Vector3>(corners[4]);
                // normal
                vertexStream.Write<Vector3>(new Vector3(-1, 0, 0));
                // uv
                vertexStream.Write<Vector2>(new Vector2(0, 0));

                //2 nd
                // pos
                vertexStream.Write<Vector3>(corners[3]);
                // normal
                vertexStream.Write<Vector3>(new Vector3(-1, 0, 0));
                // uv
                vertexStream.Write<Vector2>(new Vector2(1, 1));

                // pos
                vertexStream.Write<Vector3>(corners[7]);
                // normal
                vertexStream.Write<Vector3>(new Vector3(-1, 0, 0));
                // uv
                vertexStream.Write<Vector2>(new Vector2(0, 1));

                // pos
                vertexStream.Write<Vector3>(corners[4]);
                // normal
                vertexStream.Write<Vector3>(new Vector3(-1, 0, 0));
                // uv
                vertexStream.Write<Vector2>(new Vector2(0, 0));

                /// DOWN ///

                // pos
                vertexStream.Write<Vector3>(corners[0]);
                // normal
                vertexStream.Write<Vector3>(new Vector3(0, -1, 0));
                // uv
                vertexStream.Write<Vector2>(new Vector2(1, 0));

                // pos
                vertexStream.Write<Vector3>(corners[4]);
                // normal
                vertexStream.Write<Vector3>(new Vector3(0, -1, 0));
                // uv
                vertexStream.Write<Vector2>(new Vector2(1, 1));

                // pos
                vertexStream.Write<Vector3>(corners[1]);
                // normal
                vertexStream.Write<Vector3>(new Vector3(0, -1, 0));
                // uv
                vertexStream.Write<Vector2>(new Vector2(0, 0));

                // 2 nd
                // pos
                vertexStream.Write<Vector3>(corners[4]);
                // normal
                vertexStream.Write<Vector3>(new Vector3(0, -1, 0));
                // uv
                vertexStream.Write<Vector2>(new Vector2(1, 1));

                // pos
                vertexStream.Write<Vector3>(corners[5]);
                // normal
                vertexStream.Write<Vector3>(new Vector3(0, -1, 0));
                // uv
                vertexStream.Write<Vector2>(new Vector2(0, 1));

                // pos
                vertexStream.Write<Vector3>(corners[1]);
                // normal
                vertexStream.Write<Vector3>(new Vector3(0, -1, 0));
                // uv
                vertexStream.Write<Vector2>(new Vector2(0, 0));

                /// RIGHT ///

                // pos
                vertexStream.Write<Vector3>(corners[2]);
                // normal
                vertexStream.Write<Vector3>(new Vector3(1, 0, 0));
                // uv
                vertexStream.Write<Vector2>(new Vector2(0, 1));


                // pos
                vertexStream.Write<Vector3>(corners[1]);
                // normal
                vertexStream.Write<Vector3>(new Vector3(1, 0, 0));
                // uv
                vertexStream.Write<Vector2>(new Vector2(0, 0));


                // pos
                vertexStream.Write<Vector3>(corners[5]);
                // normal
                vertexStream.Write<Vector3>(new Vector3(1, 0, 0));
                // uv
                vertexStream.Write<Vector2>(new Vector2(1, 0));

                // 2 nd


                // pos
                vertexStream.Write<Vector3>(corners[2]);
                // normal
                vertexStream.Write<Vector3>(new Vector3(1, 0, 0));
                // uv
                vertexStream.Write<Vector2>(new Vector2(0, 1));


                // pos
                vertexStream.Write<Vector3>(corners[5]);
                // normal
                vertexStream.Write<Vector3>(new Vector3(1, 0, 0));
                // uv
                vertexStream.Write<Vector2>(new Vector2(1, 0));


                // pos
                vertexStream.Write<Vector3>(corners[6]);
                // normal
                vertexStream.Write<Vector3>(new Vector3(1, 0, 0));
                // uv
                vertexStream.Write<Vector2>(new Vector2(1, 1));

                /// TOP ///

                // pos
                vertexStream.Write<Vector3>(corners[7]);
                // normal
                vertexStream.Write<Vector3>(new Vector3(0, 1, 0));
                // uv
                vertexStream.Write<Vector2>(new Vector2(0, 1));


                // pos
                vertexStream.Write<Vector3>(corners[3]);
                // normal
                vertexStream.Write<Vector3>(new Vector3(0, 1, 0));
                // uv
                vertexStream.Write<Vector2>(new Vector2(0, 0));


                // pos
                vertexStream.Write<Vector3>(corners[2]);
                // normal
                vertexStream.Write<Vector3>(new Vector3(0, 1, 0));
                // uv
                vertexStream.Write<Vector2>(new Vector2(1, 0));

                // 2nd

                // pos
                vertexStream.Write<Vector3>(corners[7]);
                // normal
                vertexStream.Write<Vector3>(new Vector3(0, 1, 0));
                // uv
                vertexStream.Write<Vector2>(new Vector2(0, 1));

                // pos
                vertexStream.Write<Vector3>(corners[2]);
                // normal
                vertexStream.Write<Vector3>(new Vector3(0, 1, 0));
                // uv
                vertexStream.Write<Vector2>(new Vector2(1, 0));

                // pos
                vertexStream.Write<Vector3>(corners[6]);
                // normal
                vertexStream.Write<Vector3>(new Vector3(0, 1, 0));
                // uv
                vertexStream.Write<Vector2>(new Vector2(1, 1));

                /// BACK ///

                // pos
                vertexStream.Write<Vector3>(corners[6]);
                // normal
                vertexStream.Write<Vector3>(new Vector3(0, 0, 1));
                // uv
                vertexStream.Write<Vector2>(new Vector2(0, 1));

                // pos
                vertexStream.Write<Vector3>(corners[5]);
                // normal
                vertexStream.Write<Vector3>(new Vector3(0, 0, 1));
                // uv
                vertexStream.Write<Vector2>(new Vector2(0, 0));

                // pos
                vertexStream.Write<Vector3>(corners[4]);
                // normal
                vertexStream.Write<Vector3>(new Vector3(0, 0, 1));
                // uv
                vertexStream.Write<Vector2>(new Vector2(1, 0));

                // 2 nd
                // pos
                vertexStream.Write<Vector3>(corners[6]);
                // normal
                vertexStream.Write<Vector3>(new Vector3(0, 0, 1));
                // uv
                vertexStream.Write<Vector2>(new Vector2(0, 1));

                // pos
                vertexStream.Write<Vector3>(corners[4]);
                // normal
                vertexStream.Write<Vector3>(new Vector3(0, 0, 1));
                // uv
                vertexStream.Write<Vector2>(new Vector2(1, 0));

                // pos
                vertexStream.Write<Vector3>(corners[7]);
                // normal
                vertexStream.Write<Vector3>(new Vector3(0, 0, 1));
                // uv
                vertexStream.Write<Vector2>(new Vector2(1, 1));

            }



            // Create new vertex buffer

        }

        public void DrawCubes(int numCubes)
        {
            vertexStream.Position = 0;

            var vertexBuffer = SharpDX.Toolkit.Graphics.Buffer.Vertex.New(m_graphicsDeviceManager.GraphicsDevice, vertexStream);

            this.vertexSize = vertexSize;

            m_graphicsDeviceManager.GraphicsDevice.SetVertexBuffer(0, vertexBuffer, vertexSize);
            m_graphicsDeviceManager.GraphicsDevice.SetVertexInputLayout(InputLayout);


            //_graphicsDevice.DrawIndexed(mesh.PrimitiveTopology, mesh.IndexCount);
            m_graphicsDeviceManager.GraphicsDevice.Draw(SharpDX.Toolkit.Graphics.PrimitiveType.TriangleList, numCubes * 12 * 3); //change to 6 //////////////////////////////////////////////////
        }
        private void LoadLevel()
        {
            levels = new OuterCube[5];
            /// LVL 0
            List<SmallCube> cubes1 = new List<SmallCube>();
            cubes1.Add(new SmallCube(new Vector3(0.0f, -0.2f, 0.0f), 0.1f));
            levels[0] = new OuterCube(cubes1, new Vector2(0, -0.3f), Wall.Side.FRONT);
            List<SmallCube> cubes2 = new List<SmallCube>();
            cubes2.Add(new SmallCube(new Vector3(0.5f, -0.2f, 0.0f), 0.1f));
            cubes2.Add(new SmallCube(new Vector3(0.3f, -0.3f, 0.1f), 0.1f));
            cubes2.Add(new SmallCube(new Vector3(0.2f, -0.4f, 0.2f), 0.1f));
            cubes2.Add(new SmallCube(new Vector3(0.1f, -0.5f, 0.4f), 0.1f));
            cubes2.Add(new SmallCube(new Vector3(0.0f, -0.6f, 0.6f), 0.1f));
            cubes2.Add(new SmallCube(new Vector3(0.0f, -0.7f, 0.8f), 0.1f));
            levels[2] = new OuterCube(cubes2, new Vector2(0, -0.3f), Wall.Side.RIGHT);
            List<SmallCube> cubes3 = new List<SmallCube>();
            cubes3.Add(new SmallCube(new Vector3(0.5f, -0.3f, 0.5f), 0.1f));
            cubes3.Add(new SmallCube(new Vector3(0.5f, -0.2f, 0.5f), 0.1f));
            cubes3.Add(new SmallCube(new Vector3(0.5f, -0.1f, 0.5f), 0.1f));
            cubes3.Add(new SmallCube(new Vector3(0.5f, 0.0f, 0.5f), 0.1f));
            cubes3.Add(new SmallCube(new Vector3(0.5f, 0.1f, 0.5f), 0.1f));
            cubes3.Add(new SmallCube(new Vector3(0.5f, 0.2f, 0.5f), 0.1f));
            cubes3.Add(new SmallCube(new Vector3(0.5f, 0.3f, 0.5f), 0.1f));
            cubes3.Add(new SmallCube(new Vector3(0.05f, -0.3f, 0.05f), 0.1f));
            cubes3.Add(new SmallCube(new Vector3(0.05f, -0.3f, -0.05f), 0.1f));
            cubes3.Add(new SmallCube(new Vector3(-0.05f, -0.3f, 0.05f), 0.1f));
            cubes3.Add(new SmallCube(new Vector3(-0.05f, -0.3f, -0.05f), 0.1f));
            levels[1] = new OuterCube(cubes3, new Vector2(0, -0.2f), Wall.Side.RIGHT);
            List<SmallCube> cubes4 = new List<SmallCube>();
            cubes4.Add(new SmallCube(new Vector3(0.7f, 0.9f, 1.0f), 0.1f));
            cubes4.Add(new SmallCube(new Vector3(0.7f, 0.8f, 1.0f), 0.1f));
            cubes4.Add(new SmallCube(new Vector3(0.7f, 0.7f, 1.0f), 0.1f));
            cubes4.Add(new SmallCube(new Vector3(0.8f, 0.7f, 1.0f), 0.1f));
            cubes4.Add(new SmallCube(new Vector3(0.9f, 0.7f, 1.0f), 0.1f));
            cubes4.Add(new SmallCube(new Vector3(-0.3f, 0.0f, 0.0f), 0.1f));
            cubes4.Add(new SmallCube(new Vector3(-0.3f, 0.1f, -0.1f), 0.1f));
            cubes4.Add(new SmallCube(new Vector3(-0.2f, 0.1f, -0.2f), 0.1f));
            cubes4.Add(new SmallCube(new Vector3(-0.2f, 0.2f, -0.3f), 0.1f));
            cubes4.Add(new SmallCube(new Vector3(-0.1f, 0.2f, -0.4f), 0.1f));
            cubes4.Add(new SmallCube(new Vector3(-0.1f, 0.3f, -0.5f), 0.1f));
            
            levels[3] = new OuterCube(cubes4, new Vector2(0.9f, 0.9f), Wall.Side.BACK);
            List<SmallCube> cubes5 = new List<SmallCube>();
            cubes5.Add(new SmallCube(new Vector3(-0.5f, 0.0f, -0.1f), 0.1f));
            cubes5.Add(new SmallCube(new Vector3(-0.5f, 0.1f, 0.0f), 0.1f));
            cubes5.Add(new SmallCube(new Vector3(-0.5f, 0.0f, 0.1f), 0.1f));
            cubes5.Add(new SmallCube(new Vector3(-0.5f, -0.1f, 0.0f), 0.1f));
            cubes5.Add(new SmallCube(new Vector3(-0.7f, -0.6f, -0.6f), 0.1f));
            cubes5.Add(new SmallCube(new Vector3(-0.2f, 0.2f, 0.5f), 0.1f));
            cubes5.Add(new SmallCube(new Vector3(0.2f, 0.2f, 0.5f), 0.1f));
            levels[4] = new OuterCube(cubes5, new Vector2(0, 0), Wall.Side.LEFT);


            outerCube = levels[0];
        }

    }

}

